/*************************************************************
	Main.c SC02 Boot Loader 

						2015.7.17	M.Ustumi@ARCDEVICE

**************************************************************/

#include"SC02_bt.h"
#include"ESPIx_bt.h"
#include"USB_bt.h"

#define CM_MASTER_ID		0xFF

#define CM_OPEN 		1
#define CM_CLOSE 		2
#define CM_RESET 		3
#define CM_PING			4
#define CM_SET_ID 		5
#define CM_GET_ID	 	6
#define CM_GET_VER 		7
#define CM_BLINK 		8

#define RTSP_CONNECT	20
#define RTSP_ERASE		21
#define RTSP_WRITE		22
#define RTSP_READ		23
#define RTSP_RUN		24
#define RTSP_TEST		25

#define CM_PROGRAMING_MODE		30
#define CM_WRITER_MODE			31


void InternalError( void )
{
	while(1) UT_Blink(1);
}

BOOL EraseCode( void )
{
	WORD wAddr; 
	BYTE b;
	
	if(!USB_Read( (BYTE*)&wAddr, 2 )) return( FALSE );

	FLASH_Erase_32dw( wAddr );
	b = 'a';
	return( USB_Write( &b, 1 ) );
}

BOOL WriteCode( void )
{
	WORD wAddr,wSize,wWriteNum; 
	WORD w, wCode[128];
	BYTE b;
	
	if(!USB_Read( (BYTE*)&wAddr, 2 ) ) return( FALSE );
	if(!USB_Read( (BYTE*)&wSize, 2 ) ) return( FALSE );
	wWriteNum = wSize / 8;
	b = 'a';
	if(!USB_Write( &b, 1 ) ) return( FALSE );

	if(!USB_Read( (BYTE*)wCode, wSize*2 )) return( FALSE );
	
	for(w=0; w<wWriteNum; w++){
		FLASH_Write_4dw( wAddr, &wCode[w*8]  );
		wAddr += 8;
	}
	b = 'a';
	return( USB_Write( &b, 1 ) );
}

BOOL ReadCode( void )
{
	WORD wAddr,wSize,wReadNum;
	WORD w, wCode[128];
	BYTE b;

	if(!USB_Read( (BYTE*)&wAddr, 2 )) return( FALSE );
	if(!USB_Read( (BYTE*)&wSize, 2 )) return( FALSE );
	wReadNum = wSize / 2;
	
	for(w=0; w<wReadNum; w++ ){
		FLASH_Read_dw( wAddr, &wCode[w*2] );
		wAddr+=2;
	}	

	if(!USB_Write( (BYTE*)wCode, wSize*2 )) return( FALSE );

	b = 'a';
	return( USB_Write( &b, 1 ) );
}

BOOL ProgramingMode( void )
{
	BYTE bCommand;
	BOOL ret;

	LED_ON();
	ret = TRUE;
	while(ret){
		if(!USB_Read( &bCommand, 1 ) ) return( FALSE );
		switch(bCommand){
		case RTSP_ERASE: ret = EraseCode(); break;
		case RTSP_WRITE: ret = WriteCode(); break;
		case RTSP_READ: ret = ReadCode();  break;
		case RTSP_RUN: return( TRUE ); break;
		case RTSP_TEST: UT_Blink(1); break;
		
		default: UT_Blink(4); ret = FALSE; break; 
		}
		LED=~LED;
	}
	LED_ON();
	return( ret );
}

BOOL WR_EraseCode( void )
{
	WORD wAddr; 
	WORD w;
	
	if(!USB_Read( (BYTE*)&wAddr, 2 )) InternalError();

	if(!ESPI1_Write( &wAddr, 1 )) return( FALSE );
	ESPI1_WriteDisable();

	if(!ESPI1_ReadEnable()) return( FALSE );
	if(!ESPI1_Read( &w, 1 )) return( FALSE );
	ESPI1_ReadDisable();

	if(!USB_Write( (BYTE*)&w, 1 )) InternalError();

	return( TRUE );
}

BOOL WR_WriteCode( void )
{
	WORD wAddr,wSize; 
	WORD wCode[128], w;
	
	if(!USB_Read( (BYTE*)&wAddr, 2 )) InternalError();
	if(!USB_Read( (BYTE*)&wSize, 2 )) InternalError();

	if(!ESPI1_Write( &wAddr, 1 )) return( FALSE );
	if(!ESPI1_Write( &wSize, 1 )) return( FALSE );
	ESPI1_WriteDisable();

	if(!ESPI1_ReadEnable()) return( FALSE );
	if(!ESPI1_Read( &w, 1 )) return( FALSE );
	ESPI1_ReadDisable();

	if(!USB_Write( (BYTE*)&w, 1 )) InternalError();

	if(!USB_Read( (BYTE*)wCode, wSize*2 )) InternalError();

	ESPI1_WriteEnable();
	if(!ESPI1_Write( wCode, wSize )) return( FALSE );	
	ESPI1_WriteDisable();

	if(!ESPI1_ReadEnable()) return( FALSE );
	if(!ESPI1_Read( &w, 1 )) return( FALSE );
	ESPI1_ReadDisable();
	if(!USB_Write( (BYTE*)&w, 1 )) InternalError();

	return( TRUE );
}

BOOL WR_ReadCode( void )
{
	WORD wAddr,wSize;
	WORD wCode[128], w;

	if(!USB_Read( (BYTE*)&wAddr, 2 )) InternalError();
	if(!USB_Read( (BYTE*)&wSize, 2 )) InternalError();

	if(!ESPI1_Write( &wAddr, 1 )) return( FALSE );
	if(!ESPI1_Write( &wSize, 1 )) return( FALSE );
	ESPI1_WriteDisable();
	
	if(!ESPI1_ReadEnable()) return( FALSE );
	if(!ESPI1_Read( wCode, wSize )) return( FALSE );

	USB_Write( (BYTE*)wCode, wSize*2 );
	
	if(!ESPI1_Read( &w, 1 )) return( FALSE );
	ESPI1_ReadDisable();
	
	if(!USB_Write( (BYTE*)&w, 1 )) InternalError();

	return( TRUE );
}


BOOL WriterMode( void )
{
	WORD wCommand=0, w;
	BYTE bChildID, b;
	BOOL ret;

	ESPI1_Open();

	if(!USB_Read( &bChildID, 1 )) InternalError(); // qfoCXidM

	ESPI1_WriteEnable();
	if(!ESPI1_SetCommand( bChildID, CM_PROGRAMING_MODE )) return( FALSE );
	ESPI1_WriteDisable();
	
	if(!ESPI1_ReadEnable()) return( FALSE );
	if(!ESPI1_Read( &w, 1 )){
		b = 0;
		if(!USB_Write( &b, 1 )) InternalError();
		return( FALSE );
	}
	ESPI1_ReadDisable();

	b = (BYTE)w;
	if(!USB_Write( &b, 1 )) InternalError();
	if( w!=CM_PROGRAMING_MODE ) return( FALSE );

	ret = TRUE;	
	LED_ON();
	while(ret){
		if(!USB_Read( (BYTE*)&wCommand, 1 ) ) InternalError();
		ESPI1_WriteEnable();
		if(!ESPI1_Write( &wCommand, 1 )) return( FALSE );
		switch(wCommand){
		case RTSP_ERASE: ret = WR_EraseCode(); break;
		case RTSP_WRITE: ret = WR_WriteCode(); break;
		case RTSP_READ: ret = WR_ReadCode();  break;
		case RTSP_RUN: 	return( TRUE ); break;
		default: UT_Blink(4); ret = FALSE; break; 
		}
		LED = ~LED;

	}
	LED_ON();

	return( ret );
}

WORD _EEDATA(2) EEP_wID=0;	//vOݎ̏ ID

BYTE GetID( void )
{
	WORD w;

	EEP_Read( 
		__builtin_tblpage(&EEP_wID),
		__builtin_tbloffset(&EEP_wID),
	 	&w, 1 );
	return( (BYTE)w );
}

void SetID( BYTE bID )
{
	WORD w;

	w = bID;
	EEP_Write( 
		__builtin_tblpage(&EEP_wID),
		__builtin_tbloffset(&EEP_wID),
	 	&w, 1 );
}

int main(void)
{
	WORD w;
	BYTE bCommand, b, bID, bChildID;

	UT_SetTimeout( 5.0f );	// 5 sec

	USB_Open();
	
	bID = GetID();	

	LED_ON();
	TRIS_LED = 0; // output
	UT_Wait_ms(1000);
	if(!USB_DataReady() ) 	asm("GOTO 0x00100");

	if(!USB_Read( &b, 1 )) InternalError();		// id M
	if(!USB_Read( &bCommand, 1 )) InternalError();

	if( (b!=bID)&&(b!=CM_MASTER_ID )){
		b = 0;	// mode 0
		USB_Write( &b, 1 );
		asm("GOTO 0x00100");
	}
	
	switch(bCommand){
	case CM_PROGRAMING_MODE:
		UT_Blink(2);
		b = CM_PROGRAMING_MODE; // [h𑗐M
		USB_Write( &b, 1 );
		if( ProgramingMode() ) asm("GOTO 0x00100");
		break;
	case CM_WRITER_MODE:
		UT_Blink(2);
		b = CM_WRITER_MODE; // [h𑗐M
		USB_Write( &b, 1 );
		if(WriterMode()) asm("GOTO 0x00100");
		while(1) UT_Blink(2);
		break;
	case CM_SET_ID:
		b = CM_SET_ID; // [h𑗐M
		USB_Write( &b, 1 );
		if(!USB_Read( &bID, 1 )) InternalError();
		SetID( bID );
		asm("GOTO 0x00100");
		break;
	case CM_GET_ID:
		b = CM_GET_ID; // [h𑗐M
		USB_Write( &b, 1 );
		if(!USB_Write( &bID, 1 )) InternalError();
		asm("GOTO 0x00100");
		break;
	default:
		b = 0; // [h0𑗐M
		USB_Write( &b, 1 );
		break; 
	}
	while(1) UT_Blink(4);
}



