/**************************************************************
	SC02_bt.c  hardware setting and utility functions
				
						2015.7.17 M.Ustumi@ARCDEVICE

***************************************************************/

#include<stdio.h>
#include"SC02_bt.h"

inline void BUS_SetInput( void )
{
	TRISD = 0xFF; // ALL input
	TRISE = 0x0F | TRISE ; // RE0-RE3 input
}

inline void BUS_SetOutput( void )
{
	TRISD = 0xF0; // D0-D3 output
	TRISE = 0xF0 & TRISE; // D4-D7 output
}

inline BYTE BUS_Input( void )
{
	BYTE b;
	b = PORTE<<4;		
	return( (BYTE)( (b&0xF0) | (PORTD & 0x0F) ) );
}

inline void BUS_Output( BYTE b )
{
	LATD = b;
	LATE = (LATE & 0xFFF0) | (0x000F & (b >> 4));
}

void UT_SetTimeout( float dt_s  )
{
	int period;
	WORD *pw;
	DWORD dw;
	
	IEC0bits.T2IE = 0; // Timer1 interrupt disable
	IFS0bits.T2IF = 0; // flag clear
	IEC0bits.T3IE = 0; // Timer1 interrupt disable
	IFS0bits.T3IF = 0; // flag clear

	TMR2 = 0;
	TMR3 = 0;
	T3CON = 0x00; // clear
	T2CON = 0x0038; //0000 0000 0011 1000
	/*
		15:   TON   = 0 stop
		14:   N/A
		13:   TSIDL = 0 count contine in idle mode
		12-7: N/A
		6:    TGATE = 0 disable
		5-4:  TCKPS = 11 1:256 prescale
		3:    T32 = 1	32bit mode
		2:    N/A
		1:    TCS   = 0 internal clock (Fcy = Fosc/4)
		0:    N/A
	*/

	// ̌vZ 10MHzNbN
	pw = (WORD*)&dw;
	dw = (DWORD)(dt_s*1000.0f*1000.0f/(4.0f/(10.0f*8.0f)*256.0f));
	PR3 = pw[1];
	PR2 = pw[0];
}

void UT_StartTimeoutTimer( void )
{
	IFS0bits.T3IF = 0; // flag clear
	TMR2 = 0;
	TMR3 = 0;
	T2CONbits.TON = 1;
}

void UT_Wait_us( WORD us  )
{
	while( --us ){
		Nop();	Nop();	Nop();
		Nop();	Nop();	Nop();
		Nop();	Nop();	Nop();
		Nop();	Nop();	Nop();
		Nop();	Nop();	Nop();
	};
}

void UT_Wait_ms( WORD ms )
{
	while(ms--) UT_Wait_us( 1000 );
}

void UT_Blink( WORD t )
{
	while(t--){
		LED=~LED;
		UT_Wait_ms(125);
		LED=~LED;
		UT_Wait_ms(125);
	}
	UT_Wait_ms(250);
}

void UT_ZeroMemory( void* pBuf, WORD wSize )
{
	BYTE* ptr;
	ptr = (BYTE*)pBuf;
	while( wSize-- ) *ptr++=0;
}

void NVM_Update( void )
{
	asm("push SR ");
	asm("mov #0x00E0, W0 ");
	asm("ior	SR ");
	asm("mov #0x55, W0 ");
	asm("mov W0, NVMKEY ");
	asm("mov #0xAA, W0 ");
	asm("mov W0, NVMKEY	");
	NVMCONbits.WR = 1;
	asm("nop");
	asm("nop");
	while( NVMCONbits.WR );
	asm("pop SR");
}

void NVM_Write_w( WORD wPage, WORD wOffset, WORD* pwBuf )
{
	asm( "mov W0,TBLPAG " );
	asm( "tblwtl [W2], [W1] " );	
}

void NVM_Read_w( WORD wPage, WORD wOffset, WORD* pwBuf )
{ 
	asm( "mov W0,TBLPAG " );
	asm( "tblrdl [W1], [W2] " );
}


// 32߁i64wordsjPʂŏ
void FLASH_Erase_32dw( WORD wAddr )
{
	NVMADR  = wAddr;	// High & Low
	NVMADRU = 0;		// Upper
	NVMCON  = 0x4041;	// Set erase command
	NVM_Update();	
}

// 4߁i8wordsjPʂŏ
void FLASH_Write_4dw( WORD wAddr, WORD* pwCode )
{
	asm("clr w2");
	asm("mov w2, TBLPAG ");
	asm("tblwtl [W1++], [W0] ");	
	asm("tblwth [W1++], [W0++] ");
	asm("tblwtl [W1++], [W0] ");	
	asm("tblwth [W1++], [W0++]");
	asm("tblwtl [W1++], [W0] ");	
	asm("tblwth [W1++], [W0++] ");
	asm("tblwtl [W1++], [W0] ");	
	asm("tblwth [W1],   [W0] ");
	NVMCON = 0x4001;
	NVM_Update();	
}

// 1߁i2wordsjPʂœǂݍ
void FLASH_Read_dw( WORD wAddr, WORD* pwCode )
{
	asm("clr w2");
	asm("mov w2, TBLPAG ");
	asm("tblrdl [W0], [W1++] ");
	asm("tblrdh [W0], [W1] ");
}


void EEP_Read( WORD wPage, WORD wOffset, WORD* pwBuf, WORD wSize )
{
	while(wSize--){	// WORD access
		NVM_Read_w( wPage, wOffset,	pwBuf++ );
		wOffset+=2;
	}
}

void EEP_Write( WORD wPage, WORD wOffset, WORD* pwBuf, WORD wSize )
{
	while(wSize--){
		// Erace EEP 1 word
		NVMADRU = wPage;
		NVMADR  = wOffset;
		NVMCON  = 0x4044;
		NVM_Update();

		// Write EEP 1 word
		NVM_Write_w( wPage, wOffset, pwBuf++);
		NVMCON = 0x4004;
		NVM_Update();	

		wOffset+=2;
	}

}
